//=============================================================================
//
//  CLASS CSurfacePlane
//
//=============================================================================
#ifndef RAYTRACER_SURFACEPLANE_H
#define RAYTRACER_SURFACEPLANE_H

//== INCLUDES =================================================================
#include "Surface.h"

//== CLASS DEFINITION =========================================================

/*!
	A plane 
*/
class Plane : public Surface
{

    //-------------------------------------------------------------- private data
protected:
    VectorType3 m_v3Normal;
    VectorType3 m_v3Point;
    VectorType3 m_v3X, m_v3Y;

    //-------------------------------------------------------------- private methods
protected:
    void    constructONB( const VectorType3   &v,
                          VectorType3         &x,
                          VectorType3         &y,
                          VectorType3         &z ) const;


public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    Plane()
    {
    };

    //! initializing constructor
    Plane(
        const VectorType3   &v3Normal,  
        const VectorType3   &v3Point )  :
    m_v3Point( v3Point )
    {
        setNormal( v3Normal );
    };

    //-------------------------------------------------------------- element access
    void setNormal(
        const VectorType3   &v3Normal ) 
    {
        constructONB( v3Normal, m_v3X, m_v3Y, m_v3Normal );
    };

    
    const VectorType3 &getNormal() const
    {
        return m_v3Normal;
    };

    
    void setPoint(
        const VectorType3   &v3Point )  
    {
        m_v3Point = v3Point;
    };

    
    const VectorType3 &getPoint() const
    {
        return m_v3Point;
    };

    
    AABBox getAABB() const
    {
        return AABBox();
    };

    //render object in OpenGL
    void    renderGL() const;

    //translate object
    void    translate( float x, float y, float z );

    //rotate object
    void    rotate( float rotx, float roty, float rotz );

    //get Translation
    void getTranslation(
        float   *x,     
        float   *y,     
        float   *z )    
    {
        *x = m_v3Point[0];
        *y = m_v3Point[1];
        *z = m_v3Point[2];
    };

    //get Rotation
    void    getRotation( int *rotx, int *roty, int *rotz );

    //--------------------------------------------------------------- intersections
    bool    intersect( const Ray       &clRay,
                       const double  t0,
                       const double  t1,
                       TracingContext  &tContext ) const;
};

//=============================================================================
#endif // RAYTRACER_SURFACEPLANE_H

