//=============================================================================
//
//  CLASS SurfaceList
//
//=============================================================================
#ifndef RAYTRACER_SURFACELIST_H
#define RAYTRACER_SURFACELIST_H

//== INCLUDES =================================================================
#include "Types.h"
#include "Surface.h"
#include "TriangleSoup.h"
#include "Accelerator.h"
#include <list>

//== CLASS DEFINITION =========================================================

/*!
	Simple linear list of surfaces
*/

class Accelerator;
enum AcceleratorType;

typedef std::list<Surface *>    SurfaceContainerType;
typedef std::list<TriangleSoup *>    TriangleSoupContainerType;

class SurfaceList : public Surface
{

    //-------------------------------------------------------------- private types
    //-------------------------------------------------------------- private data
private:
    AABBox  m_bbox;
    bool    m_bOwnsSurfaces;
    AcceleratorType m_acceleratorType;
    Accelerator *m_accelerator;

    //-------------------------------------------------------------- constructors
public:

    //! constructor
    SurfaceList(
        bool    bOwnsSurfaces = true )  :
        m_bOwnsSurfaces( bOwnsSurfaces ),
        m_accelerator(NULL),
        m_acceleratorType(NONE)
    {
        m_bbox.Reset();
    };

    //! destructor
    ~SurfaceList()
    {
        removeAllSurfaces();
    };
    SurfaceContainerType        m_surfaceContainer;
    TriangleSoupContainerType    m_triangleSoupContainer;

    //-------------------------------------------------------------- element access
    AABBox getAABB() const
    {
        return m_bbox;
    };

    //-------------------------------------------------------------- interface
    void addSurface(
        Surface *pclSurface )   
    {
        m_surfaceContainer.push_back( pclSurface );
        m_bbox.extendBy( pclSurface->getAABB() );
    };
    
    void    addSurface( TriangleSoup *pclTriangleSoup );
    
    void    removeAllSurfaces();

    //--------------------------------------------------------------- intersections
    //Schnitt der Szene mit Primaer- und Sekundaerstrahlen
    bool    intersect( const Ray       &clRay,
                       const double  t0,
                       const double  t1,
                       TracingContext  &tContext ) const;
    
    //Schnitt der Szene mit Schattenstrahl
    //Unterscheidet sich von intersect nur bei Wahl des Lighbuffers
    bool    intersectShadow( const Ray       &clRay, 
                            const double  t0,     
                            const double  t1,     
                            TracingContext  &tContext )  const;
    
    //Render Object in OpenGL
    void    renderGL() const;

    //translate object
    void    translate( float x, float y, float z );

    //rotate object
    void    rotate( float rotx, float roty, float rotz );

    //inititalize the accelerator
    void initAccelerator( AcceleratorType acceleratorType );

    //delete accelerator after rendering
    void clearAccelerator( );

    //get Translation
    void getTranslation(
        float   *x,     
        float   *y,     
        float   *z )
    {
        *x = 0;
        *y = 0;
        *z = 0;
    };

    //get Rotation
    void getRotation(
        int *rotx,  
        int *roty,  
        int *rotz ) 
    {
        *rotx = 0;
        *roty = 0;
        *rotz = 0;
    };

    //Select next object in List
    Surface *selectNext();
};
//=============================================================================
#endif // RAYTRACER_SURFACELIST_H
