//=============================================================================
//
//  CLASS TriangleSoup
//
//=============================================================================

#ifndef RAYTRACER_TRIANGLESOUP_H
#define RAYTRACER_TRIANGLESOUP_H

//== INCLUDES =================================================================

#include "Types.h"
#include "Triangle.h"
#include "SurfaceList.h"

#include <list>

//== CLASS DEFINITION =========================================================

class SurfaceList;

/*!
	A simple triangle-mesh with obj-interface
*/

//Structure to store the Materials of an object file
struct ActMaterial
{
    Shader *m_shader;
    int m_beginFace; //start face for this material
    int m_stopFace; //end face for this material
};

typedef std::vector<VectorType3>   VertexListType;
typedef std::vector<VectorType3>   NormalListType;
typedef std::vector<Triangle *>    TriangleListType;
typedef std::vector<Triangle *>    TriangleListType;
typedef std::vector<ActMaterial *>   ActMaterialListType;

class TriangleSoup
{
    //Structure needed only temporarly to load the materials from a mtl-file
    struct MaterialNode
    {
        char m_name[1024];
        Shader *m_shader;
    };
    
//-------------------------------------------------------------- typedefs
public:

//-------------------------------------------------------------- private data
private:	

	VertexListType	    m_clVertexList;
	NormalListType	    m_clNormalList;
	TriangleListType    m_clTriangleList;
	ActMaterialListType m_clActMaterialList;

	AABBox                  m_clAABBox;	
	bool					m_bAABBComputed;
	const Shader            *m_shader;
	bool                    m_drawnThisCycle;
	VectorType3             m_lastPostition;
	bool                    m_selected;
	void computeBoundingBox();


//-------------------------------------------------------------- constructors
public:
	//!Default
	TriangleSoup();
	TriangleSoup( const char* sFilename, std::vector<Shader *>   *materialList );
	static void loadMaterials( const char* sFilename, std::vector<Shader *>   *materialList, std::list<MaterialNode> *materialNodes );

  
//-------------------------------------------------------------- element access
public:
	const VertexListType& getVertexList()  const { return m_clVertexList; };	
	const NormalListType& getNormalList()  const { return m_clNormalList; };
	const TriangleListType& getTriangleList() const { return m_clTriangleList; };

	AABBox getAABB() const { return m_clAABBox; };

//-------------------------------------------------------------- operations
public:
	void unitize();
	void unitizeMax();
	void moveTo( const VectorType3& v3Location );
	void translate( const VectorType3& v3Translation );
	void translateRelative( const VectorType3& v3Translation );
	void scale( const VectorType3& v3scale );
	void addTrianglesToScene( SurfaceList *scene );
	
	void updateNormals();
	
	void renderGL();
    
    void setShader( const Shader    *pclShader )    
    {
        m_shader = pclShader;
    };

    
    const Shader *getShader(int triangleIndex) const;
    
    bool drawnThisCycle() const
    {
        return m_drawnThisCycle;
    };
    
    void setDrawnThisCycle( bool drawn )
    {
        m_drawnThisCycle = drawn;
    };
    
    void setSelected( bool    sel )   
    {
        m_selected = sel;
        if ( m_selected )
            m_lastPostition = VectorType3( 0, 0, 0 );
    };

    bool getSelected() const
    {
        return m_selected;
    };
};
//=============================================================================
#endif // RAYTRACER_TRIANGLEMESH_H
//=============================================================================
