//=============================================================================
//
//  Main Window
//
//=============================================================================
#include <QtGui>
#include <QByteArray>
#include "window.h"
MainWindow::MainWindow()
{

    //Global settings
    createStatusBar();
    createSliders();
    createMainFrame();
    createActions();
    createMenus();
    readsettings();
}


void MainWindow::closeEvent(
    QCloseEvent * )    
{
}


void MainWindow::about()
{
    QMessageBox::about( this, tr( "About Application" ),
                        tr( "Small self-explaining Raytracer" ) );
}


void MainWindow::createActions()
{
    m_openSceneAct = new QAction( tr( "&Open Scene" ), this );
    m_openSceneAct->setShortcut( tr( "Ctrl+N" ) );
    m_openSceneAct->setStatusTip( tr( "Open Scene" ) );
    connect( m_openSceneAct, SIGNAL( triggered() ), m_raytraceArea,
             SLOT( openScene() ) );
    m_saveImageAct = new QAction( tr( "&Save Image" ), this );
    m_saveImageAct->setShortcut( tr( "Ctrl+S" ) );
    m_saveImageAct->setStatusTip( tr( "Save Image" ) );
    connect( m_saveImageAct, SIGNAL( triggered() ), m_raytraceArea,
             SLOT( saveImage() ) );
    m_exitAct = new QAction( tr( "E&xit" ), this );
    m_exitAct->setShortcut( tr( "Ctrl+Q" ) );
    m_exitAct->setStatusTip( tr( "Exit the application" ) );
    connect( m_exitAct, SIGNAL( triggered() ), this, SLOT( close() ) );
    m_renderAct = new QAction( tr( "render" ), this );
    m_renderAct->setShortcut( tr( "Ctrl+X" ) );
    m_renderAct->setStatusTip( tr( "render""clipboard" ) );
    connect( m_renderAct, SIGNAL( triggered() ), m_raytraceArea, SLOT( render() ) );
    m_aboutAct = new QAction( tr( "&About" ), this );
    m_aboutAct->setStatusTip( tr( "Show the application's About box" ) );
    connect( m_aboutAct, SIGNAL( triggered() ), this, SLOT( about() ) );
    m_aboutQtAct = new QAction( tr( "About &Qt" ), this );
    m_aboutQtAct->setStatusTip( tr( "Show the Qt library's About box" ) );
    connect( m_aboutQtAct, SIGNAL( triggered() ), qApp, SLOT( aboutQt() ) );
    connect( m_xSlider, SIGNAL( valueChanged ( int ) ), m_glArea,
             SLOT( setXTranslation ( int ) ) );
    connect( m_ySlider, SIGNAL( valueChanged ( int ) ), m_glArea,
             SLOT( setYTranslation ( int ) ) );
    connect( m_zSlider, SIGNAL( valueChanged ( int ) ), m_glArea,
             SLOT( setZTranslation ( int ) ) );
    connect( m_xRotSlider, SIGNAL( valueChanged ( int ) ), m_glArea,
             SLOT( setXRotation ( int ) ) );
    connect( m_yRotSlider, SIGNAL( valueChanged ( int ) ), m_glArea,
             SLOT( setYRotation ( int ) ) );
    connect( m_zRotSlider, SIGNAL( valueChanged ( int ) ), m_glArea,
             SLOT( setZRotation ( int ) ) );
    connect( m_glArea, SIGNAL( xTranslationChanged ( int ) ), m_xSlider,
             SLOT( setValue ( int ) ) );
    connect( m_glArea, SIGNAL( yTranslationChanged ( int ) ), m_ySlider,
             SLOT( setValue ( int ) ) );
    connect( m_glArea, SIGNAL( zTranslationChanged ( int ) ), m_zSlider,
             SLOT( setValue ( int ) ) );
    connect( m_glArea, SIGNAL( xRotationChanged ( int ) ), m_xRotSlider,
             SLOT( setValue ( int ) ) );
    connect( m_glArea, SIGNAL( yRotationChanged ( int ) ), m_yRotSlider,
             SLOT( setValue ( int ) ) );
    connect( m_glArea, SIGNAL( zRotationChanged ( int ) ), m_zRotSlider,
             SLOT( setValue ( int ) ) );
    connect( m_selectNextObjectButton, SIGNAL( clicked() ), m_glArea,
             SLOT( selectNextObject() ) );
    connect( m_renderSceneButton, SIGNAL( clicked() ), m_raytraceArea,
             SLOT( render() ) );
    connect( m_setRayMode, SIGNAL( stateChanged ( int ) ), m_glArea,
             SLOT( setRayMode ( int ) ) );
	connect( m_setDisplayUpdate, SIGNAL( stateChanged ( int ) ), m_raytraceArea,
             SLOT( setDisplayMode ( int ) ) );
    connect( m_raytraceArea, SIGNAL( setGLScene( Raytracer *,
             SurfaceList * ) ), m_glArea, SLOT( setScene(
                 Raytracer *, SurfaceList * ) ) );
    connect( m_raytraceArea, SIGNAL( renderRaysGL( RayArray * ) ),
             m_glArea, SLOT( renderRays( RayArray * ) ) );
    connect( m_raytraceArea, SIGNAL( resetGL() ), m_glArea, SLOT( reset() ) );
    connect( m_raytraceArea, SIGNAL( startRaytracingGL( QImage * ) ), m_glArea, SLOT( startRaytracing( QImage * ) ) );
	
    connect( m_setImageSize, SIGNAL( clicked() ), this,
             SLOT( setImageSize() ) );
    connect( this, SIGNAL( updateImageSizeRT( int ) ), m_raytraceArea,
             SLOT( updateImageSize( int ) ) );
    connect( m_setRecursionDepth, SIGNAL( clicked() ), this,
             SLOT( setRecursionDepth() ) );
    connect( this, SIGNAL( updateRecursionDepthRT( int ) ), m_raytraceArea,
             SLOT( updateRecursionDepth( int ) ) );
    connect( m_choseAccelerator, SIGNAL( activated ( const QString & ) ), m_raytraceArea,
             SLOT( setAccelerator( const QString & ) ) );   
}

void MainWindow::setImageSize()
{
	QString input = m_imageSizeEdit->text();
	int newsize = input.toInt();
	if( newsize > 0 )
		emit updateImageSizeRT( newsize );
}

void MainWindow::setRecursionDepth()
{
	QString input = m_recursionDepthEdit->text();
	int newsize = input.toInt();
	if( newsize > 0 )
		emit updateRecursionDepthRT( newsize );
}

void MainWindow::createMenus()
{
    m_fileMenu = menuBar()->addMenu( tr( "&Load/Save File" ) );
    m_fileMenu->addAction( m_openSceneAct );
    m_fileMenu->addSeparator();
    m_fileMenu->addAction( m_saveImageAct );
    m_fileMenu->addSeparator();
    m_fileMenu->addAction( m_exitAct );
    m_editMenu = menuBar()->addMenu( tr( "&Action" ) );
    m_editMenu->addAction( m_renderAct );
    menuBar()->addSeparator();
    m_helpMenu = menuBar()->addMenu( tr( "&Help" ) );
    m_helpMenu->addAction( m_aboutAct );
    m_helpMenu->addAction( m_aboutQtAct );
}


void MainWindow::createSliders()
{
    m_xSlider = createSlider( -100, 100 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QLabel  *labelx = new QLabel( "x", m_xSlider );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_ySlider = createSlider( -100, 100 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QLabel  *labely = new QLabel( "y", m_ySlider );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_zSlider = createSlider( -100, 100 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QLabel  *labelz = new QLabel( "z", m_zSlider );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_xRotSlider = createSlider( 0, 360 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QLabel  *labelxRot = new QLabel( "x-Rot", m_xRotSlider );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_yRotSlider = createSlider( 0, 360 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QLabel  *labelyRot = new QLabel( "y-Rot", m_yRotSlider );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_zRotSlider = createSlider( 0, 360 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QLabel  *labelzRot = new QLabel( "z-Rot", m_zRotSlider );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_xSlider->setValue( 0 );
    m_ySlider->setValue( 0 );
    m_zSlider->setValue( 0 );
    m_xRotSlider->setValue( 180 );
    m_yRotSlider->setValue( 180 );
    m_zRotSlider->setValue( 180 );
    m_selectNextObjectButton = new QPushButton( "Select next Object" );
    m_selectNextObjectButton->setMaximumWidth( 200 );
    m_renderSceneButton = new QPushButton( "Raytrace" );
    m_renderSceneButton->setMaximumWidth( 200 );
    m_setRayMode = new QCheckBox;
	m_setDisplayUpdate  = new QCheckBox;
	m_imageSizeEdit = new QLineEdit;
	m_imageSizeEdit->setMaximumWidth( 200 );
	m_setImageSize = new QPushButton( "Set Image Size" );
	m_setImageSize->setMaximumWidth( 200 );
	m_recursionDepthEdit = new QLineEdit;
	m_recursionDepthEdit->setMaximumWidth( 200 );
	m_setRecursionDepth = new QPushButton( "Set Recursion Depth" );
	m_setRecursionDepth->setMaximumWidth( 200 );
	
	m_choseAccelerator  = new QComboBox( false );
	m_choseAccelerator->addItem( "No Acceleration" );
	m_choseAccelerator->addItem( "Grid" );
	m_choseAccelerator->addItem( "KD-Tree" );
	m_choseAccelerator->addItem( "AABB-Tree" );
	m_choseAccelerator->addItem( "Lightbuffer" );
}


void MainWindow::createStatusBar()
{
    statusBar()->showMessage( tr( "Ready" ) );
}


void MainWindow::createMainFrame()
{
    m_raytraceArea = new RaytracingArea;
    m_glArea = new GLWidget;

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QFrame      *frame = new QFrame;
    QGroupBox   *allGroupBox = new QGroupBox();
    QHBoxLayout *allLayout = new QHBoxLayout();
    QHBoxLayout *raytraceLayout = new QHBoxLayout();
    QGroupBox   *raytraceGroupBox = new QGroupBox( tr( "Raytraced" ) );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    raytraceLayout->addWidget( m_raytraceArea );
    raytraceGroupBox->setLayout( raytraceLayout );


    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QHBoxLayout *glLayout = new QHBoxLayout();
    QGroupBox   *glGroupBox = new QGroupBox( tr( "OpenGL" ) );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    glLayout->addWidget( m_glArea );
    glGroupBox->setLayout( glLayout );
    allLayout->addWidget( glGroupBox );
    allLayout->addWidget( raytraceGroupBox );
    allGroupBox->setLayout( allLayout );

	allLayout->setStretchFactor( glGroupBox, 1 );
	allLayout->setStretchFactor( raytraceGroupBox, 1 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
	QGroupBox   *interactionGroupBox = new QGroupBox( tr( "Interaction" ) );
	QHBoxLayout *interactionLayout = new QHBoxLayout;
    QGroupBox   *sliderGroupBox = new QGroupBox( tr( "Move Objects" ) );
    QHBoxLayout *sliderLayout = new QHBoxLayout;
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    sliderLayout->addWidget( m_xSlider );
    sliderLayout->addWidget( m_ySlider );
    sliderLayout->addWidget( m_zSlider );
    sliderLayout->addWidget( m_xRotSlider );
    sliderLayout->addWidget( m_yRotSlider );
    sliderLayout->addWidget( m_zRotSlider );
    sliderGroupBox->setLayout( sliderLayout );
    interactionLayout->addWidget( sliderGroupBox );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QGroupBox   *buttonGroupBox = new QGroupBox( tr( "Actions" ) );
    //QVBoxLayout *buttonLayout = new QVBoxLayout;
	QGridLayout *buttonLayout = new QGridLayout;
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

	QLabel  *labelray = new QLabel( "Show rays:", m_setRayMode );
    QLabel  *labelDisplay = new QLabel( "Enable Timing", m_setDisplayUpdate );
	QLabel  *labelImageSize = new QLabel( "Image Width and Height in #Pixels:", m_imageSizeEdit );
	QLabel  *labelRecursionDepth = new QLabel( "Recursion Depth:", m_recursionDepthEdit );

    buttonLayout->addWidget( m_selectNextObjectButton, 0, 0 );
    buttonLayout->addWidget( labelray, 0, 2 );
	buttonLayout->addWidget( m_setRayMode, 0, 3 );
    buttonLayout->addWidget( labelImageSize, 1, 0 );
	buttonLayout->addWidget( m_imageSizeEdit, 1, 1 );
	buttonLayout->addWidget( m_setImageSize, 1, 2 );
    buttonLayout->addWidget( labelRecursionDepth, 2, 0 );
	buttonLayout->addWidget( m_recursionDepthEdit, 2, 1 );
	buttonLayout->addWidget( m_setRecursionDepth, 2, 2 );
    buttonLayout->addWidget( m_renderSceneButton, 3, 0 );
    buttonLayout->addWidget( labelDisplay, 3, 2 );
	buttonLayout->addWidget( m_setDisplayUpdate, 3, 3 );
	buttonLayout->addWidget( m_choseAccelerator, 4, 0 );
	
    buttonGroupBox->setLayout( buttonLayout );
    interactionLayout->addWidget( buttonGroupBox );
	interactionGroupBox->setLayout( interactionLayout );

	interactionLayout->setStretchFactor( buttonGroupBox, 1 );
	interactionLayout->setStretchFactor( sliderGroupBox, 1 );

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QVBoxLayout *mainLayout = new QVBoxLayout;
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    mainLayout->addWidget( allGroupBox );
    mainLayout->addWidget( interactionGroupBox );
	mainLayout->setStretchFactor( allGroupBox, 2 );
	mainLayout->setStretchFactor( interactionGroupBox, 1 );
    frame->setLayout( mainLayout );
    setCentralWidget( frame );
}


void MainWindow::readsettings()
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QSettings   settings( "Raytracing", "Application Example" );
    QPoint      pos = settings.value( "pos", QPoint( 200, 200 ) ).toPoint();
    QSize       size = settings.value( "size", QSize( 800, 800 ) ).toSize();
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    resize( size );
    move( pos );
}


QSlider *MainWindow::createSlider(
    int min,    
    int max )   
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    QSlider *slider = new QSlider( Qt::Vertical );
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    slider->setRange( min, max );
    slider->setSingleStep( 1 );
    slider->setPageStep( 15 );
    slider->setTickInterval( 15 );
    slider->setTickPosition( QSlider::TicksRight );
    return slider;
}
